@@ -399,6 +399,10 @@ fr_command_zoo_new (FRProcess         *process,
 {
 	FRCommand *comm;
 
+	if (!is_program_in_path("zoo")) {
+		return NULL;
+	}	
+
 	comm = FR_COMMAND (g_object_new (FR_TYPE_COMMAND_ZOO, NULL));
 	fr_command_construct (comm, process, filename);
 
